/* principio del ejemplo base base.c */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <cstdlib>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
//#include <cairo.h>
#include <string>

#include "gloobus-preview-interface-main.h"
//#include "iImage.h"
//#include "iText.h"
//#include "iMusic.h"
//#include "iDocument.h"
//#include "iMovie.h"
//#include "iList.h"

#include "gloobus-preview-window.h"
#include "gloobus-preview-defines.h"
#include "gloobus-preview-singleton.h"






using namespace std;


//GLOBAL VARIABLES TO STORE PLUGINS INFORMATION
vector<plugin_filetype> pluginVector;				          		  	//VECTOR TO STORE WICH FILETYPE READ EACH PLUGIN

map <string, create_t_image  *, less<string> > factory_image;		  	//Global variable that plugin uses to register
map <string, destroy_t_image *, less<string> > factoryDestroy_image;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_music  *, less<string> > factory_music;		  	//Global variable that plugin uses to register
map <string, destroy_t_music *, less<string> > factoryDestroy_music;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_movie  *, less<string> > factory_movie;		  	//Global variable that plugin uses to register
map <string, destroy_t_movie *, less<string> > factoryDestroy_movie;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_text   *, less<string> > factory_text;		  	//Global variable that plugin uses to register
map <string, destroy_t_text  *, less<string> > factoryDestroy_text;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_document   *, less<string> > factory_document;		  	//Global variable that plugin uses to register
map <string, destroy_t_document  *, less<string> > factoryDestroy_document;   //MAP WITH NAME AND DESTRUCTOR

map <string, create_t_list   *, less<string> > factory_list;		  	//Global variable that plugin uses to register
map <string, destroy_t_list  *, less<string> > factoryDestroy_list;   	//MAP WITH NAME AND DESTRUCTOR

map <string, create_t_free   *, less<string> > factory_free;		  	//Global variable that plugin uses to register
map <string, destroy_t_free  *, less<string> > factoryDestroy_free;   	//MAP WITH NAME AND DESTRUCTOR


void destroy   	( GtkWidget *widget, gpointer data );
void key_press 	( GtkWidget *widget, GdkEventKey *event);
void log_handler(const gchar *log_domain, GLogLevelFlags log_level,const gchar *message, gpointer user_data);



// ======================================= MAIN ===================================== //
int main (int argc, char *argv[])
{

	//Variables
	string 		filename;


	// GTK INITIALIZATION	
	if (!g_thread_supported ()){ g_thread_init(NULL); }
	gdk_threads_init();
    gdk_threads_enter();
	
	gtk_init (&argc, &argv);

	//Set log function 	

	//Set debug
	bool debug = false;
	if(argc == 3 && strcmp(argv[2],"--debug") == 0) {debug=true;};
	if(argc == 2 && strcmp(argv[1],"--debug") == 0) {debug=true;};

	g_log_set_handler (NULL, (GLogLevelFlags)(G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
                     | G_LOG_FLAG_RECURSION), log_handler, (void*)debug);

	if(debug) g_message("MODE DEBUG: ON\n");
	

	// Obtain filename to preview from the clipboard
	if(argc==1 || (argc==2 && debug==true))				
	{
		g_debug("Getting file from clipboard\n");
		GtkClipboard *clip  =  gtk_clipboard_get 		   		( GDK_SELECTION_CLIPBOARD );
		gchar **uri 		=  gtk_clipboard_wait_for_uris 		( clip );
		if(uri == NULL)
		{
		 	   g_critical	( "NO FILE IN THE CLIPBOARD\n" );
		 	   exit			( -1 );
		}
		else
		{	
			string temp = uri[0];
			temp = temp.substr(7,temp.length());
			temp = g_filename_from_uri(uri[0], NULL, NULL);	//Replaces the strange characters
			g_debug("Filename: %s\n", filename.c_str());
			g_debug("ClipBoard: %s\n",temp.c_str());
			filename = temp;
		}

	}
	// Obtain filename to preview from command line
	else 
	{ 	

		filename = argv[1];										// Filename from command line
		if (filename.c_str()[0] != '/') 
		{
			filename = "./" + filename;
		}
		if (filename.c_str()[filename.size()-1] == '/') 			// Final slash crashes
		{
			filename = filename.substr(0,filename.size()-1);
		}
		
	}
	

	gloobusWin *win3 = new gloobusWin();
	win3->create_window	( DEFAULT_WIDTH, DEFAULT_HEIGHT );
	
	//Events Handlers
	g_signal_connect (win3->get_window_pointer(),"key-press-event", 		//And this one, catchs key events
					 G_CALLBACK(key_press), NULL);							//to see when space has being pressed, and quit
					 	
				Gloobus::instance()->init( filename ,win3 );
				Gloobus::instance()->create_plugin();
	int w 	  =	Gloobus::instance()->get_width ()+ SHADOW_WIDTH*2;					
	int h 	  =	Gloobus::instance()->get_height()+ SHADOW_WIDTH*2 + BOTTOM_HEIGHT+HEADER_HEIGHT;		//Its the shadow
	
	string title =	filename;
	
				Gloobus::instance()->draw_plugin ();

	win3->set_size		( w, h);
	win3->set_title		( title );
	win3->add_common_buttons ();				//Add close and gloobus button
 	win3->show_all		();
 	

	//============================= Main bucle ============================ //
	
	    
    gtk_main ();
    
	//DESTROY OBJECTS BEFORE END
	
	//Destroy created plugin

	Gloobus::instance()->destroy_plugin();			//Destroy current plugin
	Gloobus::instance()->end();						//Unload All plugins before leave
	gdk_threads_leave();
    return 0;
}

//============================= OTHER FUNCTIONS ============================ //


// ===================================== DESTROY ===================================== //
// This function is called when a exit event is recived
void destroy ( GtkWidget *widget, gpointer data )
{
    gtk_main_quit ();
}

// ===================================== KEY_PRESS =================================== //
// This function is called when there is a key press event
void key_press ( GtkWidget *widget, GdkEventKey *event)
{
    g_message ("Key Pressed %s (%i)\n",gdk_keyval_name(event->keyval),event->keyval);
	if(event->keyval == 32 || event->keyval == 65307)		//Space or ESC
	{
		gtk_main_quit();
	}
		
	if(event->keyval == 102)								//F key
		Gloobus::instance()->fullscreen();
		
	if(event->keyval == 65363)				//Right Key
		Gloobus::instance()->next_file();

	
	if(event->keyval == 65361)				//Left key
		Gloobus::instance()->prev_file();
		
	if(event->keyval == 65293){				//Enter key
		Gloobus::instance()->launch_default_application();
		gtk_main_quit();
	}
}

// =================================== LOG_HANDLER =================================== //
void log_handler  (const gchar *log_domain,GLogLevelFlags log_level, const gchar *message, gpointer debug)
{

	if(!(bool)debug && G_LOG_LEVEL_DEBUG == log_level)
	{
	}
	else
	{
		char* add = "";

		if (log_level==  G_LOG_LEVEL_MESSAGE)
		{
			add = "\033[1;37m[INFO]\033[1;m";		//White
		}
		if (log_level == G_LOG_LEVEL_DEBUG)
		{
			add = "\033[1;34m[DEBUG]\033[1;m";		//Blue
		}
		if (log_level==  G_LOG_LEVEL_WARNING)
		{
			add = "\033[1;33m[WARNING]\033[1;m";	//YELLOW
		}
		if (log_level==  G_LOG_LEVEL_CRITICAL)
		{
			add = "\033[1;31m[CRITICAL]\033[1;m";	//RED
		}

		g_print("%s %s",add,message);
	}
}


